package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.exception.AlreadyExistsException
import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import javax.persistence.*
import javax.validation.constraints.Min
import javax.validation.constraints.NotNull

@Entity
@Table(name = "mix_material")
data class MixMaterial(
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    override val id: Long?,

    @ManyToOne
    @JoinColumn(name = "material_id")
    val material: Material,

    var quantity: Float,

    var position: Int
) : Model

data class MixMaterialDto(
    val materialId: Long,

    @field:Min(0, message = VALIDATION_SIZE_GE_ZERO)
    val quantity: Float,

    val position: Int
)

data class MixMaterialOutputDto(
    val id: Long,
    val material: MaterialOutputDto,
    val quantity: Float,
    val position: Int
)

// ==== DSL ====
fun mixMaterial(
    id: Long? = null,
    material: Material = material(),
    quantity: Float = 0f,
    position: Int = 0,
    op: MixMaterial.() -> Unit = {}
) = MixMaterial(id, material, quantity, position).apply(op)

fun mixMaterialDto(
    materialId: Long = 0L,
    quantity: Float = 0f,
    position: Int = 0,
    op: MixMaterialDto.() -> Unit = {}
) = MixMaterialDto(materialId, quantity, position).apply(op)

// ==== Exceptions ====
private const val MIX_MATERIAL_NOT_FOUND_EXCEPTION_TITLE = "Mix material not found"
private const val MIX_MATERIAL_ALREADY_EXISTS_EXCEPTION_TITLE = "Mix material already exists"
private const val MIX_MATERIAL_EXCEPTION_ERROR_CODE = "mixmaterial"

fun mixMaterialIdNotFoundException(id: Long) =
    NotFoundException(
        MIX_MATERIAL_EXCEPTION_ERROR_CODE,
        MIX_MATERIAL_NOT_FOUND_EXCEPTION_TITLE,
        "A mix material with the id $id could not be found",
        id
    )

fun mixMaterialIdAlreadyExistsException(id: Long) =
    AlreadyExistsException(
        MIX_MATERIAL_EXCEPTION_ERROR_CODE,
        MIX_MATERIAL_ALREADY_EXISTS_EXCEPTION_TITLE,
        "A mix material with the id $id already exists",
        id
    )
